@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2011 The OpenLDAP Foundation.
+ * Copyright 2000-2018 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -163,8 +163,8 @@ static ConfigOCs bdbocs[] = {
 #endif
 		"SUP olcDatabaseConfig "
 		"MUST olcDbDirectory "
-		"MAY ( olcDbCacheSize $ olcDbCheckpoint $ olcDbConfig $ "
-		"olcDbCryptFile $ olcDbCryptKey $ "
+		"MAY ( olcDbCacheSize $ olcDbCheckpoint $ olcDbChecksum $ "
+		"olcDbConfig $ olcDbCryptFile $ olcDbCryptKey $ "
 		"olcDbNoSync $ olcDbDirtyRead $ olcDbIDLcacheSize $ "
 		"olcDbIndex $ olcDbLinearIndex $ olcDbLockDetect $ "
 		"olcDbMode $ olcDbSearchStack $ olcDbShmKey $ "
@@ -241,6 +241,8 @@ bdb_online_index( void *ctx, void *arg )
 		rc = TXN_BEGIN( bdb->bi_dbenv, NULL, &txn, bdb->bi_db_opflags );
 		if ( rc ) 
 			break;
+		Debug( LDAP_DEBUG_TRACE, LDAP_XSTRING(bdb_online_index) ": txn id: %x\n",
+			txn->id(txn), 0, 0 );
 		if ( getnext ) {
 			getnext = 0;
 			BDB_ID2DISK( id, &nid );
@@ -282,17 +284,16 @@ bdb_online_index( void *ctx, void *arg )
 		}
 		if ( ei->bei_e ) {
 			rc = bdb_index_entry( op, txn, BDB_INDEX_UPDATE_OP, ei->bei_e );
-			if ( rc == DB_LOCK_DEADLOCK ) {
+			if ( rc ) {
 				TXN_ABORT( txn );
-				ldap_pvt_thread_yield();
-				continue;
-			}
-			if ( rc == 0 ) {
-				rc = TXN_COMMIT( txn, 0 );
-				txn = NULL;
-			}
-			if ( rc )
+				if ( rc == DB_LOCK_DEADLOCK ) {
+					ldap_pvt_thread_yield();
+					continue;
+				}
 				break;
+			}
+			rc = TXN_COMMIT( txn, 0 );
+			txn = NULL;
 		}
 		id++;
 		getnext = 1;
@@ -323,32 +324,39 @@ bdb_cf_cleanup( ConfigArgs *c )
 {
 	struct bdb_info *bdb = c->be->be_private;
 	int rc = 0;
-
-	if ( bdb->bi_flags & BDB_UPD_CONFIG ) {
-		if ( bdb->bi_db_config ) {
-			int i;
-			FILE *f = fopen( bdb->bi_db_config_path, "w" );
-			if ( f ) {
-				for (i=0; bdb->bi_db_config[i].bv_val; i++)
-					fprintf( f, "%s\n", bdb->bi_db_config[i].bv_val );
-				fclose( f );
-			}
-		} else {
-			unlink( bdb->bi_db_config_path );
-		}
-		bdb->bi_flags ^= BDB_UPD_CONFIG;
-	}
+	BerVarray bva;
 
 	if ( bdb->bi_flags & BDB_DEL_INDEX ) {
 		bdb_attr_flush( bdb );
 		bdb->bi_flags ^= BDB_DEL_INDEX;
 	}
-	
+
 	if ( bdb->bi_flags & BDB_RE_OPEN ) {
 		bdb->bi_flags ^= BDB_RE_OPEN;
+		bva = bdb->bi_db_config;
+		bdb->bi_db_config = NULL;
 		rc = c->be->bd_info->bi_db_close( c->be, &c->reply );
-		if ( rc == 0 )
+		if ( rc == 0 ) {
+			if ( bdb->bi_flags & BDB_UPD_CONFIG ) {
+				if ( bva ) {
+					int i;
+					FILE *f = fopen( bdb->bi_db_config_path, "w" );
+					if ( f ) {
+						bdb->bi_db_config = bva;
+						bva = NULL;
+						for (i=0; bdb->bi_db_config[i].bv_val; i++)
+							fprintf( f, "%s\n", bdb->bi_db_config[i].bv_val );
+						fclose( f );
+					} else {
+						ber_bvarray_free( bva );
+					}
+				} else {
+					unlink( bdb->bi_db_config_path );
+				}
+				bdb->bi_flags ^= BDB_UPD_CONFIG;
+			}
 			rc = c->be->bd_info->bi_db_open( c->be, &c->reply );
+		}
 		/* If this fails, we need to restart */
 		if ( rc ) {
 			slapd_shutdown = 2;
@@ -556,7 +564,7 @@ bdb_cf_gen( ConfigArgs *c )
 				for (; bdb->bi_db_config[i].bv_val; i++)
 					bdb->bi_db_config[i] = bdb->bi_db_config[i+1];
 			}
-			bdb->bi_flags |= BDB_UPD_CONFIG;
+			bdb->bi_flags |= BDB_UPD_CONFIG|BDB_RE_OPEN;
 			c->cleanup = bdb_cf_cleanup;
 			break;
 		/* Doesn't really make sense to change these on the fly;
@@ -594,10 +602,11 @@ bdb_cf_gen( ConfigArgs *c )
 			if ( c->valx == -1 ) {
 				int i;
 
-				/* delete all (FIXME) */
+				/* delete all */
 				for ( i = 0; i < bdb->bi_nattrs; i++ ) {
 					bdb->bi_attrs[i]->ai_indexmask |= BDB_INDEX_DELETING;
 				}
+				bdb->bi_defaultmask = 0;
 				bdb->bi_flags |= BDB_DEL_INDEX;
 				c->cleanup = bdb_cf_cleanup;
 
@@ -758,7 +767,7 @@ bdb_cf_gen( ConfigArgs *c )
 		}
 
 		if ( bdb->bi_flags & BDB_IS_OPEN ) {
-			bdb->bi_flags |= BDB_UPD_CONFIG;
+			bdb->bi_flags |= BDB_UPD_CONFIG|BDB_RE_OPEN;
 			c->cleanup = bdb_cf_cleanup;
 		} else {
 		/* If we're just starting up...
