@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2012 The OpenLDAP Foundation.
+ * Copyright 2000-2018 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -391,14 +391,16 @@ mdb_idl_fetch_key(
 
 int
 mdb_idl_insert_keys(
+	BackendDB	*be,
 	MDB_cursor	*cursor,
 	struct berval *keys,
 	ID			id )
 {
+	struct mdb_info *mdb = be->be_private;
 	MDB_val key, data;
 	ID lo, hi, *i;
 	char *err;
-	int	rc, k;
+	int	rc = 0, k;
 	unsigned int flag = MDB_NODUPDATA;
 #ifndef	MISALIGNED_OK
 	int kbuf[2];
@@ -490,8 +492,8 @@ mdb_idl_insert_keys(
 				}
 			} else {
 			/* There's room, just store it */
-				if ( slapMode & SLAP_TOOL_QUICK )
-					flag |= MDB_APPEND;
+				if (id == mdb->mi_nextid)
+					flag |= MDB_APPENDDUP;
 				goto put1;
 			}
 		} else {
@@ -503,9 +505,17 @@ mdb_idl_insert_keys(
 			if ( id < lo || id > hi ) {
 				/* position on lo */
 				rc = mdb_cursor_get( cursor, &key, &data, MDB_NEXT_DUP );
+				if ( rc != 0 ) {
+					err = "c_get lo";
+					goto fail;
+				}
 				if ( id > hi ) {
 					/* position on hi */
 					rc = mdb_cursor_get( cursor, &key, &data, MDB_NEXT_DUP );
+					if ( rc != 0 ) {
+						err = "c_get hi";
+						goto fail;
+					}
 				}
 				data.mv_size = sizeof(ID);
 				data.mv_data = &id;
@@ -518,7 +528,7 @@ mdb_idl_insert_keys(
 			}
 		}
 	} else if ( rc == MDB_NOTFOUND ) {
-		flag &= ~MDB_APPEND;
+		flag &= ~MDB_APPENDDUP;
 put1:	data.mv_data = &id;
 		data.mv_size = sizeof(ID);
 		rc = mdb_cursor_put( cursor, &key, &data, flag );
@@ -542,11 +552,12 @@ fail:
 
 int
 mdb_idl_delete_keys(
+	BackendDB	*be,
 	MDB_cursor	*cursor,
 	struct berval *keys,
 	ID			id )
 {
-	int	rc, k;
+	int	rc = 0, k;
 	MDB_val key, data;
 	ID lo, hi, tmp, *i;
 	char *err;
@@ -700,18 +711,11 @@ mdb_idl_intersection(
 	}
 
 	/* If a range completely covers the list, the result is
-	 * just the list. If idmin to idmax is contiguous, just
-	 * turn it into a range.
+	 * just the list.
 	 */
 	if ( MDB_IDL_IS_RANGE( b )
-		&& MDB_IDL_RANGE_FIRST( b ) <= MDB_IDL_RANGE_FIRST( a )
-		&& MDB_IDL_RANGE_LAST( b ) >= MDB_IDL_RANGE_LAST( a ) ) {
-		if (idmax - idmin + 1 == a[0])
-		{
-			a[0] = NOID;
-			a[1] = idmin;
-			a[2] = idmax;
-		}
+		&& MDB_IDL_RANGE_FIRST( b ) <= MDB_IDL_FIRST( a )
+		&& MDB_IDL_RANGE_LAST( b ) >= MDB_IDL_LLAST( a ) ) {
 		goto done;
 	}
 
@@ -1063,7 +1067,7 @@ mdb_idl_sort( ID *ids, ID *tmp )
 			ids[l+1] = ids[j];
 			ids[j] = a;
 			jstack += 2;
-			if (ir-i+1 >= j-1) {
+			if (ir-i+1 >= j-l) {
 				istack[jstack] = ir;
 				istack[jstack-1] = i;
 				ir = j-1;
@@ -1081,7 +1085,7 @@ mdb_idl_sort( ID *ids, ID *tmp )
 /* 8 bit Radix sort + insertion sort
  * 
  * based on code from http://www.cubic.org/docs/radix.htm
- * with improvements by mbackes@symas.com and hyc@symas.com
+ * with improvements by ebackes@symas.com and hyc@symas.com
  *
  * This code is O(n) but has a relatively high constant factor. For lists
  * up to ~50 Quicksort is slightly faster; up to ~100 they are even.
