@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2005-2015 The OpenLDAP Foundation.
+ * Copyright 2005-2018 The OpenLDAP Foundation.
  * Portions Copyright 2005-2006 SysNet s.n.c.
  * All rights reserved.
  *
@@ -442,6 +442,7 @@ dds_op_add( Operation *op, SlapReply *rs )
 			sc->sc_response = dds_counter_cb;
 			sc->sc_private = di;
 			sc->sc_next = op->o_callback;
+			sc->sc_writewait = 0;
 
 			op->o_callback = sc;
 		}
@@ -476,6 +477,7 @@ dds_op_delete( Operation *op, SlapReply *rs )
 			sc->sc_cleanup = dds_freeit_cb;
 			sc->sc_response = dds_counter_cb;
 			sc->sc_private = di;
+			sc->sc_writewait = 0;
 			sc->sc_next = op->o_callback;
 	
 			op->o_callback = sc;
@@ -877,6 +879,72 @@ dds_op_rename( Operation *op, SlapReply *rs )
 	return SLAP_CB_CONTINUE;
 }
 
+/* entryTtl update for client */
+static int
+dds_response( Operation *op, SlapReply *rs )
+{
+	slap_overinst	*on = (slap_overinst *)op->o_bd->bd_info;
+	dds_info_t	*di = on->on_bi.bi_private;
+	int		rc;
+
+	if ( !DDS_OFF( di )
+	     && rs->sr_type == REP_SEARCH
+	     && attr_find( rs->sr_entry->e_attrs, slap_schema.si_ad_entryTtl ) )
+	{
+		BerVarray		vals = NULL;
+		struct lutil_tm		tm;
+		struct lutil_timet	tt;
+		char			ttlbuf[STRLENOF("31557600") + 1];
+		struct berval		ttlvalue;
+		time_t			ttl;
+		int			len;
+
+		/* User already has access to entryTtl, skip ACL checks on
+		 * entryExpireTimestamp */
+		rc = backend_attribute( op, NULL, &rs->sr_entry->e_nname,
+			ad_entryExpireTimestamp, &vals, ACL_NONE );
+		if ( rc != LDAP_SUCCESS ) {
+			return rc;
+		}
+
+		assert( vals[0].bv_val[vals[0].bv_len] == '\0' );
+		if ( lutil_parsetime( vals[0].bv_val, &tm ) ) {
+			goto done;
+		}
+
+		lutil_tm2time( &tm, &tt );
+		ttl = tt.tt_sec - op->o_time;
+		ttl = (ttl < 0) ? 0 : ttl;
+		assert( ttl <= DDS_RF2589_MAX_TTL );
+
+		len = snprintf( ttlbuf, sizeof(ttlbuf), "%ld", ttl );
+		if ( len < 0 )
+		{
+			goto done;
+		}
+		ttlvalue.bv_val = ttlbuf;
+		ttlvalue.bv_len = len;
+
+		rs_entry2modifiable( op, rs, on );
+
+		if ( attr_delete( &rs->sr_entry->e_attrs,
+				slap_schema.si_ad_entryTtl ) )
+		{
+			goto done;
+		}
+		if ( attr_merge_normalize_one( rs->sr_entry,
+				slap_schema.si_ad_entryTtl,
+				&ttlvalue, op->o_tmpmemctx ) )
+		{
+			goto done;
+		}
+
+done:;
+		ber_bvarray_free_x( vals, op->o_tmpmemctx );
+	}
+	return SLAP_CB_CONTINUE;
+}
+
 static int
 slap_parse_refresh(
 	struct berval	*in,
@@ -1740,18 +1808,6 @@ dds_db_open(
 		goto done;
 	}
 
-	/* ... if there are dynamic objects, delete those expired */
-	if ( di->di_num_dynamicObjects > 0 ) {
-		/* force deletion of expired entries... */
-		be->bd_info = (BackendInfo *)on->on_info;
-		rc = dds_expire( thrctx, di );
-		be->bd_info = (BackendInfo *)on;
-		if ( rc != LDAP_SUCCESS ) {
-			rc = 1;
-			goto done;
-		}
-	}
-
 	/* start expire task */
 	ldap_pvt_thread_mutex_lock( &slapd_rq.rq_mutex );
 	di->di_expire_task = ldap_pvt_runqueue_insert( &slapd_rq,
@@ -1939,6 +1995,7 @@ dds_initialize()
 	dds.on_bi.bi_op_modify = dds_op_modify;
 	dds.on_bi.bi_op_modrdn = dds_op_rename;
 	dds.on_bi.bi_extended = dds_op_extended;
+	dds.on_response = dds_response;
 
 	dds.on_bi.bi_cf_ocs = dds_ocs;
 
